#!/bin/bash

# Train a DDAG multiclass classifier for six different human actions, then
# test the resulting classifier on the validation set.
#
# The training can take some time.  What is shown while training is mostly the
# stump search (the lines with [40000]), and the current training status after
# adding stumps (loss function on training set and realized margin).
#
# The data comes from extracting spatio-temporal interest points on the KTH
# human action classification dataset, which are preprocessed using PCA and
# k-means clustering into a 768 word codebook.  Each video is then represented
# as a sequence of sets of spatio-temporal video words.
#
# For details, see the following reference.
#
# "Discriminative Subsequence Mining for Action Classification", ICCV 2007,
#   Sebastian Nowozin, Gkhan BakIr and Koji Tsuda.
#
# Note: while the classifier is trained, some MPS linear programming problem
# files are outputted as "lp-debug-<n>.mps", where n is the LPBoost iteration.
# If something goes wrong (unlikely), you can inspect/solve them using GLPK
# with: glpsol --max --mps lp-debug-30.mps

../../static/pboost.x86 --exploration-max=200000 \
	--conv-eps=0.1 --nu=0.2 control-train.txt.cb-768.tb-15
../../static/ptest.x86 output.txt control-val.txt.cb-768.tb-15

