function [classifier, cfun] = sboost (S, Y, nu, conv_eps, htype);

% 1.5/2-class sequence LPBoosting
%
% Input
%    S: (1,n) cellarray of sequences
%    Y: (n,1) labels in {-1,1}
%    nu: LPBoost nu parameter
%    conv_eps (optional): LPBoost convergence threshold. (default: 0.05)
%    htype (optional): 1 for 1.5-class LPBoost, 2 for 2-class LPBoost
%       (default: 2);
%
% Output
%    classifier: Classifier structure with internal information, subsequences
%       and weights, see lpboost.m documentation for more information.
%    cfun: classification function, see lpboost.m

if nargin < 4
	conv_eps = 0.05;
end

if nargin < 5
	htype = 2;
end

disp(['=== SBOOST = BEGIN = sequence based LPBoosting ===']);
[classifier, cfun] = lpboost (S, Y, conv_eps, nu, ...
	@(X, Y, u, beta, max_col) findhypothesis_seq (X, Y, u, beta, max_col, htype), ...
	htype, 1);	% 1: max_col, increase to converge faster
disp(['=== END ===']);


