
#ifndef	LPBOOST_H
#define	LPBOOST_H

#include <vector>
#include <iostream>

#include <lp_lib.h>

#include <Sequence.h>
#include <PrefixSpanLPBoost.h>

namespace LPBoost
{

class
LPBoost
{
private:
	double convergenceEpsilon;

	double nu;
	unsigned int pricingCount;
	unsigned int minsup;
	unsigned int minlen;
	unsigned int maxlen;

	/* Validation set.
	 */
	bool doValidation;
	std::vector<const SeqMine::Sequence*> Sval;
	std::vector<double> Yval;

	void PrintCurrentTrainingStatus (std::vector<double>& Ylabels) const;

public:
	std::vector<double> alpha;
	std::vector<std::pair<SeqMine::SequenceT, double> > stumps;

	LPBoost (double nu, unsigned int pricingCount,
		unsigned int minsup, unsigned int minlen = 0, unsigned int maxlen = 0)
		: nu (nu), pricingCount (pricingCount), minsup (minsup),
			minlen (minlen), maxlen (maxlen)
	{
		convergenceEpsilon = 1e-4;	// TODO: make configurable
		doValidation = false;
	}

	void BuildClassifier (std::vector<const SeqMine::Sequence*>& S,
		std::vector<double>& Ylabels);

	/* Set a validation set to evaluate the error on after each boosting
	 * iteration.
	 */
	void SetValidationSet (std::vector<const SeqMine::Sequence*>& Sval,
		std::vector<double>& Yval);

	/* Compute the validation score.
	 */
	double ValidationLoss (void) const;
};

};

#endif


